/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Soll dem Datensatz mit maximalen Stundenwerte und dem Zeitstempel dieser
 * Werte erzeugen. Wenn der Aggregationsintervall laenger als eine Stunde ist,
 * dann erzeugt nur Datensaetze mit Werten auf "nicht ermittelbar" gestellt.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AggregatorSpizenStunde extends AbstraktAggregator {

	/**
	 * ATG String Konstante.
	 */
	private static final String ATG_LZD_SPITZENSTUNDE = "atg.langZeitDatenSpitzenStunde";

	/**
	 * Erweitert die Klasse LZZDaten um den spizenAnzahl der bearbeiteten
	 * Attribute.
	 */
	private class LZZSpiztenDaten extends LZZDaten {
		/**
		 * Spitzenwert fuer jedem Attribut aus fzAttrib.
		 */
		private final long[] spizenAnzahl = new long[AbstraktAggregator.FZ_ATTRIB.length];

		/**
		 * Standardkonstruktor.
		 */
		LZZSpiztenDaten() {
			resetDaten();
		}

		/**
		 * Initialisiert die Daten.
		 */
		@Override
		public void resetDaten() {
			for (int i = 0; i < spizenAnzahl.length; i++) {
				spizenAnzahl[i] = 0;
			}
		}

		/**
		 * Ob der erzeugungsintervall eine Stunde ist.
		 */
		private boolean datenErzeugungEineStunde = true;

		/**
		 * Erfragt den SpizenAnzahl der bearbeiteten Attribute.
		 *
		 * @return spizenAnzahl
		 */
		public long[] getSpizenAnzahl() {
			return spizenAnzahl;
		}

		/**
		 * Legt fest, ob die daten ermittelbar sind.
		 *
		 * @return true, wenn ermittelbar
		 */
		public boolean istErzeugungsIntervallStunde() {
			return datenErzeugungEineStunde;
		}

		/**
		 * Legt fest, ob die daten ermittelbar sind.
		 *
		 * @param ermittelbar
		 *            der Status der Ermittelbarkeit
		 */
		public void setErzeugungsIntervallStunde(final boolean ermittelbar) {
			this.datenErzeugungEineStunde = ermittelbar;
		}
	}

	@Override
	public DataDescription init(final Aspect asp, final ClientDavInterface dav,
			final Collection<SystemObject> objekteLZZ) {

		resetLzzDaten();

		final DataDescription dd = new DataDescription(
				dav.getDataModel().getAttributeGroup(AggregatorSpizenStunde.ATG_LZD_SPITZENSTUNDE), asp);

		for (final SystemObject lzz : objekteLZZ) {
			final LZZDaten lzzStruct = new LZZSpiztenDaten();
			final Data data = dav
					.createData(dav.getDataModel().getAttributeGroup(AggregatorSpizenStunde.ATG_LZD_SPITZENSTUNDE));
			lzzStruct.setErgebnissDatenSatz(data);
			setLeer(data);
			addLzzDaten(lzz, lzzStruct);
		}
		return dd;
	}

	@Override
	public void bearbeiteDatenSatz(final SystemObject objekt, final Data datensatz, final long zeitStempel) {

		if (datensatz == null) {
			return;
		}
		final LZZSpiztenDaten lzz = (LZZSpiztenDaten) getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatensatz = lzz.getErgebnissDatenSatz();
		if (ergebnissDatensatz == null) {
			return;
		}
		lzz.incrementAnzahlDatenSaetze();
		final long[] spizenAnzahl = lzz.getSpizenAnzahl();

		for (int i = 0; i < AbstraktAggregator.FZ_ATTRIB.length; i++) {
			final Data subData = datensatz.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i]);
			final Data zeitErgebnissData = ergebnissDatensatz.getItem(AbstraktAggregator.FZ_ATTRIB[i]);
			final Data wertErgebnissData = ergebnissDatensatz.getItem("q" + AbstraktAggregator.FZ_ATTRIB[i]);

			if (datensatz.getTimeValue("T").getMillis() != AbfrPufferKonstanten.STUNDE_IN_MS) {
				lzz.setErzeugungsIntervallStunde(false);
			} else if (spizenAnzahl[i] < subData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue()) {

				spizenAnzahl[i] = subData.getUnscaledValue(AbstraktAggregator.STR_WERT).longValue();
				ersetzeNeueSpitzenStunde(spizenAnzahl[i], zeitStempel, subData, wertErgebnissData, zeitErgebnissData);
			}
		}
	}

	/**
	 * Ersetzt die bisherige SpizenStunde mit der neuen.
	 *
	 * @param max
	 *            Anzahl der FZ in letzter Stunde
	 * @param zeitStempel
	 *            ZeitStempel des letzten DS
	 * @param subData
	 *            Der neue Datensatz mit Wert und Guete
	 * @param wertErgebnissData
	 *            Der DateStaz mit Wert und Guete
	 * @param zeitErgebnissData
	 *            Der DateStaz mit Zeitstempel
	 */
	public void ersetzeNeueSpitzenStunde(final long max, final long zeitStempel, final Data subData,
			final Data wertErgebnissData, final Data zeitErgebnissData) {

		zeitErgebnissData.getTimeValue(AbstraktAggregator.STR_WERT).setMillis(zeitStempel);
		wertErgebnissData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(max);

		final Data dataGuete = subData.getItem(AbstraktAggregator.STR_GUETE);
		final Data ergebnissGuete = wertErgebnissData.getItem(AbstraktAggregator.STR_GUETE);
		ergebnissGuete.getUnscaledValue(AbstraktAggregator.STR_INDEX)
		.set(dataGuete.getUnscaledValue(AbstraktAggregator.STR_INDEX).longValue());
		ergebnissGuete.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN)
		.set(dataGuete.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).intValue());

		final Data zeitGuete = zeitErgebnissData.getItem(AbstraktAggregator.STR_GUETE);
		zeitGuete.getUnscaledValue(AbstraktAggregator.STR_INDEX)
		.set(dataGuete.getUnscaledValue(AbstraktAggregator.STR_INDEX).longValue());
		zeitGuete.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN)
		.set(dataGuete.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).intValue());
	}

	@Override
	public void erzeugeErgebniss(final SystemObject objekt, final long zeitStempelVon, final long zeitStempelBis,
			final boolean ermittelbar) {

		final LZZSpiztenDaten lzz = (LZZSpiztenDaten) getLzzDatenForSystemObject(objekt);
		final Data ergebnissDatenSatz = lzz.getErgebnissDatenSatz();

		if (!ermittelbar) {
			setNichtErmittelbar(ergebnissDatenSatz);
		} else if (!lzz.istErzeugungsIntervallStunde()) {
			setNichtErmittelbar(ergebnissDatenSatz);
		}

		if (lzz.getZeitStempelLetztesDS() >= zeitStempelVon) {
			setLeer(ergebnissDatenSatz);
			lzz.resetDaten();
			lzz.setAnzahlDatenSaetze(0);
			return;
		}

		final ResultData result = new ResultData(objekt, getDdAggregierteLzd(), zeitStempelVon, ergebnissDatenSatz,
				true);
		sendeDatenSatz(objekt, result);

		setLeer(ergebnissDatenSatz);
		lzz.setAnzahlDatenSaetze(0);
		lzz.resetDaten();
		lzz.setZeitStempelLetztesDS(zeitStempelVon);
	}

	@Override
	public void setLeer(final Data data) {
		for (final String element : AbstraktAggregator.FZ_ATTRIB) {
			Data subData = data.getItem(element);
			subData.getTimeValue(AbstraktAggregator.STR_WERT).setMillis(AbfrPufferKonstanten.ZEIT_BEGINN);
			Data gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(-1);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);

			subData = data.getItem("q" + element);
			subData.getUnscaledValue(AbstraktAggregator.STR_WERT).set(-1);
			gueteData = subData.getItem(AbstraktAggregator.STR_GUETE);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_INDEX).set(-1);
			gueteData.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN).set(0);
		}
	}
}
