/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;
import java.util.Date;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.abfrpuffer.SystemDatenEndgeraet.SystemDeStatus;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Besorgt, dass die Achslastdaten regelmaesig abgefragt werden.
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class AxlDeVerwaltung extends AbstraktDeVerwaltung {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Datenbeschreibung für AXL-LZD Empfang.
	 */
	private static DataDescription ddAxlDatenEmpfang;
	/**
	 * Datenbeschreibung für Abrufe von AXL-LZD senden.
	 */
	private static DataDescription ddAxlAbrufSend;
	/**
	 * Tabelle mit Assoziationen von AchslastMessStellen SystemObjekten zum
	 * Datenengeraeten.
	 */
	private final Map<String, String> mapMSzumDE;
	/**
	 * Liste aller MessStellen, auf deren Daten wir angemeldet sind.
	 */
	private final Collection<SystemObject> archivierteMessStellen;
	/**
	 * Liste aller MessStellen, auf die wir uns aufmelden sollen.
	 */
	private final Collection<SystemObject> zurAnmeldungMessStellen;
	/**
	 * Liste aller MessStellen, die wir abmelden sollen.
	 */
	private final Collection<SystemObject> zurAbmeldungMessStellen;
	/**
	 * Datenbeschreibung von AXL-DE Betriebsparameter.
	 */
	private static DataDescription ddAxlParam;
	/**
	 * Datenbeschreibung von AXL-DE Negative Quittung.
	 */
	private static DataDescription ddAxlNegQuit;
	/**
	 * Ob der Empfang und Versand Telegramme zu DEs angemeldet ist.
	 */
	private boolean deEmfangVersandAngemeldet;

	/**
	 * Standardkonstruktor.
	 *
	 * @param verwaltungHaupt
	 *            Hauptlklasse des Moduls
	 * @param steuerModule
	 *            Liste der verwateten Steuermodule
	 */
	public AxlDeVerwaltung(final AbfragePufferdaten verwaltungHaupt, final Collection<Station> steuerModule) {
		super(verwaltungHaupt, steuerModule);
		mapMSzumDE = new Hashtable<>();
		archivierteMessStellen = new LinkedList<>();
		zurAbmeldungMessStellen = new LinkedList<>();
		zurAnmeldungMessStellen = new LinkedList<>();

		AxlDeVerwaltung.ddAxlParam = verwaltungHaupt.erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_AXLBETRIEBSPARAM, AbfrPufferKonstanten.ASP_PARAMETERSOLL,
				DataDescription.NO_SIMULATION_VARIANT_SET);
		AxlDeVerwaltung.ddAxlAbrufSend = verwaltungHaupt.erzeugeDatenBeschreibung(AbfrPufferKonstanten.ATG_TLS_AXLABRUF,
				AbfrPufferKonstanten.ASP_TLS_ABRUF, DataDescription.NO_SIMULATION_VARIANT_SET);
		AxlDeVerwaltung.ddAxlDatenEmpfang = verwaltungHaupt.erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG, AbfrPufferKonstanten.ASP_TLS_ANTWORT,
				DataDescription.NO_SIMULATION_VARIANT_SET);
		AxlDeVerwaltung.ddAxlNegQuit = verwaltungHaupt.erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_AXLNEGATIVE_QUITTUNG, AbfrPufferKonstanten.ASP_TLS_ANTWORT,
				DataDescription.NO_SIMULATION_VARIANT_SET);

		Station.setAxlDeVerwaltung(this);
	}

	/**
	 * Erste Anmeldung fuer Daten Empfang und Sendung.
	 */
	@Override
	public void ersteAnmeldung() {
		Collection<SystemObject> axlDEs;

		axlDEs = Station.getAlleAxlDEs(getSteuerModule());
		getDav().subscribeReceiver(getVerwaltungHaupt(), axlDEs, AxlDeVerwaltung.ddAxlParam, ReceiveOptions.normal(),
				ReceiverRole.receiver());
		getDav().subscribeReceiver(getVerwaltungHaupt(), axlDEs, AxlDeVerwaltung.ddAxlNegQuit, ReceiveOptions.normal(),
				ReceiverRole.receiver());

		axlDEs = Station.getAlleAxlSystemDEs(getSteuerModule());
		try {
			getDav().subscribeSender(getVerwaltungHaupt(), axlDEs, AxlDeVerwaltung.ddAxlAbrufSend, SenderRole.sender());
			deEmfangVersandAngemeldet = true;
		} catch (final OneSubscriptionPerSendData e) {
			AxlDeVerwaltung.LOGGER.error("Anmeldung für Axl-Abrufe Sender unmöglich:" + e.getMessage());
			getVerwaltungHaupt().getnachrichtenSender().sendMessage(MessageType.SYSTEM_DOMAIN, MessageGrade.FATAL,
					"Anmeldung für Axl-Abrufe Sender unmöglich:" + e.getMessage());
		}
	}

	/**
	 * Anmeldung fuer Daten Empfang und Sendung.
	 */
	@Override
	public void anmeldung() {

		streamBasierteArchivAbfrage();

		getDav().subscribeReceiver(getVerwaltungHaupt(), zurAnmeldungMessStellen, AxlDeVerwaltung.ddAxlDatenEmpfang,
				ReceiveOptions.delayed(), ReceiverRole.receiver());
		getDav().unsubscribeReceiver(getVerwaltungHaupt(), zurAbmeldungMessStellen, AxlDeVerwaltung.ddAxlDatenEmpfang);

		archivierteMessStellen.addAll(zurAnmeldungMessStellen);
		archivierteMessStellen.removeAll(zurAbmeldungMessStellen);

		for (final SystemObject fs : zurAbmeldungMessStellen) {
			final DatenEndgeraet de = Station.getDatenEndgeraet(mapMSzumDE.get(fs.getPid()));
			if (de != null) {
				final Station st = de.getStation();
				aktualisiereStatus(st, "");
			}
		}

		zurAbmeldungMessStellen.clear();
		zurAnmeldungMessStellen.clear();
	}

	/**
	 * Aktualisiert den Status der Station, nach Ankunft von Betriebsmeldungen,
	 * FehlerMeldungen, oder ArchivDaten.
	 */
	@Override
	public void aktualisiereStatus(final Station st, final String s) {
		final SystemDeStatus status = st.getSystemAxlDe().getStatus();

		if (st.istAxlOnline()) {
			if (st.istAxlParametriert()) {
				st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
				statusAktualisierungAusgabe(st, s);
				generiereAbfrage(st);
			} else {
				st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_NICHT_PARAMETRIERT);
				if (status != st.getSystemAxlDe().getStatus()) {
					statusAktualisierungAusgabe(st, s);
				}
			}
		} else {
			st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_NICHT_VERBUNDEN);
			if (status != st.getSystemAxlDe().getStatus()) {
				statusAktualisierungAusgabe(st, s);
			}
		}
	}

	/**
	 * Abmeldung fuer Daten Empfang und senden.
	 */
	@Override
	public void abmeldung() {
		getDav().unsubscribeReceiver(getVerwaltungHaupt(), archivierteMessStellen, AxlDeVerwaltung.ddAxlDatenEmpfang);
		Collection<SystemObject> axlDEs;

		if (deEmfangVersandAngemeldet) {
			axlDEs = Station.getAlleAxlSystemDEs(getSteuerModule());
			getDav().unsubscribeSender(getVerwaltungHaupt(), axlDEs, AxlDeVerwaltung.ddAxlAbrufSend);
			axlDEs = Station.getAlleAxlDEs(getSteuerModule());
			getDav().unsubscribeReceiver(getVerwaltungHaupt(), axlDEs, AxlDeVerwaltung.ddAxlParam);
			deEmfangVersandAngemeldet = false;
		}
	}

	/**
	 * Aktualisiert den Zeitstempell wenn Datei FG1 LZD vom Fahrstreifen
	 * angekommen sind.
	 *
	 * @param datei
	 *            Die empfangene Datei
	 */
	@Override
	public void aktualisiereDeZeitStemepel(final ResultData datei) {
		final SystemObject ms = datei.getObject();
		final Data daten = datei.getData();
		final DataState ds = datei.getDataState();
		long zeit = 0;
		if (daten == null) {
			// nichts zu tun
			return;
		} else if (ds.equals(DataState.DATA)) {
			zeit = datei.getDataTime();
			final String pidDe = mapMSzumDE.get((ms.getPid()));

			final DatenEndgeraet de = Station.getDatenEndgeraet(pidDe);
			final Station st = de.getStation();

			final long anzahl = de.decrementAnzahlErwarteteDatenSaetze();
			de.updateLetzterDatensatzZeitStempel(zeit);

			if (anzahl < 0) {
				AxlDeVerwaltung.LOGGER.warning(
						"Unerwartetes AXL Datum von " + de.getSoDE().getPid() + " eingekommen, fehlende DS:" + anzahl);
			}
			final long jetzt = currentTimeMillis();

			final boolean allesGekommen = st.sindAlleAxlDatenSaetzeGekommen(jetzt);

			if (allesGekommen && (st.getSystemAxlDe().getStatus() == SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT)) {
				if ((st.getAxlMaxAnzahlFehlendenDatenSaetzen(jetzt) > 0) && !st.antwortAxlNegQuittBekommen()) {
					st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
					generiereAbfrage(st);
				} else {
					st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
					st.axlTimeoutAbbrechen();
				}
			}
		}
	}

	@Override
	public void bearbeiteNegativeQuittung(final ResultData datei) {
		final SystemObject axlDe = datei.getObject();
		final DatenEndgeraet de = Station.getDatenEndgeraet(axlDe.getPid());
		if (de instanceof SystemDatenEndgeraet) {
			final SystemDatenEndgeraet sysDe = (SystemDatenEndgeraet) de;
			sysDe.erwarteKeineAntwort(true);
			sysDe.getStation().axlTimeoutAbbrechen();
		} else if (de != null) {
			// Negative Quittung von einzelnen DE
			de.setAnzahlErwarteteDatenSaetze(0);
			de.setAntwortetNegQuitt(true);

			final Station st = de.getStation();
			final long jetzt = currentTimeMillis();
			final boolean allesGekommen = st.sindAlleAxlDatenSaetzeGekommen(jetzt);

			// Wenn von anderen DEs schon alles angekommen ist
			if (allesGekommen && (st.getSystemAxlDe().getStatus() == SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT)) {
				st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH);
				st.axlTimeoutAbbrechen();
			}
		}
	}

	/**
	 * Liest die Betriebsparameter eines AxlDe aus den empfangenen Daten.
	 *
	 * @param datei
	 *            Empfangene Datei
	 */
	@Override
	public void setBetriebsParameter(final ResultData datei) {
		final SystemObject de = datei.getObject();
		final Data daten = datei.getData();
		if (daten != null) {
			final int zeitCode = daten.getUnscaledValue(AbfrPufferKonstanten.ATR_INTERVALL_AXL).intValue();

			final long zeit = AbfrPufferKonstanten.convertAxlZeitCodeZurZeit(zeitCode);

			final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(de.getPid());
			final Station st = datenEndgeraet.getStation();

			final boolean warParametriert = st.istAxlParametriert();
			datenEndgeraet.setErzeugungsIntervall(zeit);
			final boolean istParametriert = st.istAxlParametriert();

			if (warParametriert != istParametriert) {
				aktualisiereStatus(st, "Betriebsparameter eingekommen");
			}
		}
	}

	/**
	 * Generiert eine LZD Abfrage.
	 *
	 * @param st
	 *            Die Station fuer die die Abfrage geruft werden soll
	 */
	@Override
	public void generiereAbfrage(final Station st) {

		// System.out.println("Generiere Abfrage: " + st + ", " + new
		// Date(st.getAeltestenAxlZeitStempel()));
		if (st.getSystemAxlDe() == null) {
			return;
		}
		// eine Sekunde Mehr, damit wir nicht die selben Daten nochmal abfragen
		final long zeitStempelLetzterDaten = st.getAeltestenAxlZeitStempel() + 1000L + (60L * 60L);
		final long jetzt = currentTimeMillis();

		long anzahl = st.getAxlMaxAnzahlFehlendenDatenSaetzen(jetzt);

		if (st.axlFlackert(jetzt)) {
			AxlDeVerwaltung.LOGGER.warning("Lve Station flackert, Abfrage nicht erlaubt");
			return;
		} else if (!st.istAxlParametriert() || !st.istAxlOnline()) {
			AxlDeVerwaltung.LOGGER.error("Daten Abfrage nicht moeglich : Parametriert:" + st.istAxlParametriert()
			+ " Online:" + st.istAxlOnline() + " Status = " + st.getSystemAxlDe().getStatus());
			return;
		} else if (st.getSystemAxlDe().getStatus() != SystemDeStatus.DE_STAT_ABFRAGE_MOEGLICH) {
			AxlDeVerwaltung.LOGGER.error("Daten Abfrage nicht moeglich : Fehlende DatenSaetze = " + anzahl
					+ " Status = " + st.getSystemAxlDe().getStatus());
			return;
		} else if ((anzahl == 0)
				|| (getVerwaltungHaupt().getAbrufIntervallAxl() < st.getAxlDatenErzeugungsIntervallMinimum())) {
			AxlDeVerwaltung.LOGGER.warning("Axl Abfrage nicht noetig, daten aktuell: AbrufINtervall"
					+ getVerwaltungHaupt().getAbrufIntervallAxl() + " Erzuegungintervall: "
					+ st.getAxlDatenErzeugungsIntervallMinimum() + " Zeit" + new Date(jetzt));
			return;
		}
		// maximaler Datensatzanzahl begrentzt
		if (anzahl > AbfrPufferKonstanten.MAX_DATENSAETZE) {
			anzahl = AbfrPufferKonstanten.MAX_DATENSAETZE;
		}

		st.setErwarteAxlDatensaetze(anzahl);
		final long timeout = st.getAxlDeListe().size() * anzahl * AbfrPufferKonstanten.BASIS_TIMEOUT_AXL;

		generiereAbfrage(st.getSystemAxlDe().getSoDE(), zeitStempelLetzterDaten, anzahl);
		st.getSystemAxlDe().setStatus(SystemDeStatus.DE_STAT_ABFRAGE_LAEUFT);
		st.getSystemAxlDe().setLetzterAbruf(zeitStempelLetzterDaten);
		st.setAxlTimeoutWecker(timeout);

		abfrageAusgabe(st, anzahl, jetzt, timeout);

	}

	/**
	 * Generiert eine LZD Abfrage.
	 *
	 * @param systemDe
	 *            Das De mit Kanal 255, dem man abfragen soll
	 * @param zeitStempel
	 *            Zeit, aus der wir die Daten benoetigen
	 * @param anzahl
	 *            Anzahl der benoetigten Datensaetze
	 */
	@Override
	public void generiereAbfrage(final SystemObject systemDe, final long zeitStempel, final long anzahl) {
		final Data data = getDav()
				.createData(getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_TLS_AXLABRUF));
		final Data zeitData = data.getItem(AbfrPufferKonstanten.ZEITPUNKT);
		zeitData.asTimeValue().setMillis(zeitStempel);
		final Data anzahlData = data.getItem(AbfrPufferKonstanten.ATR_ANZAHL);
		anzahlData.asUnscaledValue().set(anzahl);
		final long zeit = currentTimeMillis();
		final ResultData rd = new ResultData(systemDe, AxlDeVerwaltung.ddAxlAbrufSend, zeit, data);

		try {
			getDav().sendData(rd);
			// System.out.println("AXL-Abruf: " + rd);
		} catch (final DataNotSubscribedException e) {
			AxlDeVerwaltung.LOGGER.error(e.getMessage());
			e.printStackTrace();
		} catch (final SendSubscriptionNotConfirmed e) {
			AxlDeVerwaltung.LOGGER.error(e.getMessage());
			e.printStackTrace();
		}
	}

	@Override
	public void bearbeiteNeueEinstellung(final DataIdentification dateiID, final Data neueDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbfrPufferKonstanten.TYP_ACHSLAST_MESSSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
				.equals(AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG)
				&& dateiID.getDataDescription().getAspect().getPid().equals(AbfrPufferKonstanten.ASP_TLS_ANTWORT))) {
			return;
		}

		boolean archiviert = true;

		// Wenn es nich Archiviert werden soll
		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			archiviert = false;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_ACHSLASTMESSSTELLE),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_AXLMESSSTELLENQUELLE).asReferenceValue()
				.getValueText();

		// FS assozieiert mit DEs die nicht verwaltet werden interessieren uns
		// nicht
		if (!Station.enthaeltEndgeraet(dePid)) {
			return;
		}

		if (!mapMSzumDE.containsKey(dateiID.getObject().getPid())) {
			mapMSzumDE.put(dateiID.getObject().getPid(), dePid);
		}

		zurAnmeldungMessStellen.add(dateiID.getObject());

		final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);
		datenEndgeraet.setArchiviert(archiviert);
		datenEndgeraet.setSoAssoziiert(dateiID.getObject());
		aktualisiereStatus(datenEndgeraet.getStation(), "Parametrierung");
	}

	@Override
	public void bearbeiteGeloeschteEinstellung(final DataIdentification dateiID, final Data alteDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbfrPufferKonstanten.TYP_ACHSLAST_MESSSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
				.equals(AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG)
				&& dateiID.getDataDescription().getAspect().getPid().equals(AbfrPufferKonstanten.ASP_TLS_ANTWORT))) {
			return;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_ACHSLASTMESSSTELLE),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_AXLMESSSTELLENQUELLE).asReferenceValue()
				.getValueText();

		// FS assozieiert mit DEs die nicht verwaltet werden interessieren uns
		// nicht
		if (!Station.enthaeltEndgeraet(dePid)) {
			return;
		}

		if (!mapMSzumDE.containsKey(dateiID.getObject().getPid())) {
			mapMSzumDE.put(dateiID.getObject().getPid(), dePid);
		}

		zurAbmeldungMessStellen.add(dateiID.getObject());

		final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);
		datenEndgeraet.setArchiviert(false);
		aktualisiereStatus(datenEndgeraet.getStation(), "Parametrierung");

	}

	@Override
	public void bearbeiteVeraenderteEinstellung(final DataIdentification dateiID, final Data alteDatei,
			final Data neueDatei) {

		if (!(dateiID.getObject().getType().getPid().equals(AbfrPufferKonstanten.TYP_ACHSLAST_MESSSTELLE)
				&& dateiID.getDataDescription().getAttributeGroup().getPid()
				.equals(AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG)
				&& dateiID.getDataDescription().getAspect().getPid().equals(AbfrPufferKonstanten.ASP_TLS_ANTWORT))) {
			return;
		}

		boolean neuArchiviert = true;
		boolean altArchiviert = true;

		// Wenn es nich Archiviert werden soll
		if (neueDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			neuArchiviert = false;
		}
		if (alteDatei.getUnscaledValue(AbfrPufferKonstanten.ATR_ARCHIVIEREN).getState().getValue() != 1) {
			altArchiviert = false;
		}

		final Data confDaten = dateiID.getObject().getConfigurationData(
				getDav().getDataModel().getAttributeGroup(AbfrPufferKonstanten.ATG_ACHSLASTMESSSTELLE),
				getDav().getDataModel().getAspect(AbfrPufferKonstanten.ASP_EIGENSCHAFTEN));

		final String dePid = confDaten.getItem(AbfrPufferKonstanten.ATR_AXLMESSSTELLENQUELLE).asReferenceValue()
				.getValueText();

		// FS assozieiert mit DEs die nicht verwaltet werden interessieren uns
		// nicht
		if (!Station.enthaeltEndgeraet(dePid)) {
			return;
		}

		if (!mapMSzumDE.containsKey(dateiID.getObject().getPid())) {
			mapMSzumDE.put(dateiID.getObject().getPid(), dePid);
		}

		final DatenEndgeraet de = Station.getDatenEndgeraet(dePid);
		if (altArchiviert != de.isArchiviert()) {
			AxlDeVerwaltung.LOGGER.error("Archivierung status nicht korrekt " + de.getSoDE().getPid());
		}

		if (neuArchiviert != altArchiviert) {
			if (neuArchiviert) {
				zurAnmeldungMessStellen.add(dateiID.getObject());
			} else {
				zurAbmeldungMessStellen.add(dateiID.getObject());
			}

			final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(dePid);
			datenEndgeraet.setArchiviert(neuArchiviert);
			aktualisiereStatus(datenEndgeraet.getStation(), "Parametrierung");
		}
	}

	/**
	 * Sendet eine StreamAbfrage an das Archivsystem.
	 */
	@Override
	public void streamBasierteArchivAbfrage() {
		final DataDescription datenBeschreibung = getVerwaltungHaupt().erzeugeDatenBeschreibung(
				AbfrPufferKonstanten.ATG_TLS_AXLERGEBNIS_MELDUNG, AbfrPufferKonstanten.ASP_TLS_ANTWORT,
				DataDescription.NO_SIMULATION_VARIANT_SET);
		final long jetzt = currentTimeMillis();
		getVerwaltungHaupt().streamAbfrage(this, zurAnmeldungMessStellen, datenBeschreibung, true, 1, jetzt);
	}

	@Override
	public void bearbeiteArchivDaten(final SystemObject objekt, final DataDescription datenBeschreibung,
			final DataState status, final Data daten, final long zeitStempel) {

		final String depid = mapMSzumDE.get(objekt.getPid());
		final DatenEndgeraet datenEndgeraet = Station.getDatenEndgeraet(depid);
		if (datenEndgeraet != null) {
			if (status == DataState.END_OF_ARCHIVE) {
				if (datenEndgeraet.isNochNieDaten()) {
					datenEndgeraet.updateLetzterDatensatzZeitStempel(AbfrPufferKonstanten.ZEIT_BEGINN);
					datenEndgeraet.setArchivAbgefragt(true);
					final Station st = datenEndgeraet.getStation();
					aktualisiereStatus(st, "Archiv Parametrierung");
					return;
				}
				final boolean warArchivAngefragt = datenEndgeraet.isArchivAbgefragt();
				datenEndgeraet.setArchivAbgefragt(true);
				final Station st = datenEndgeraet.getStation();
				if (!warArchivAngefragt) {
					aktualisiereStatus(st, "Archiv Parametrierung");
				}
			} else {
				datenEndgeraet.updateLetzterDatensatzZeitStempel(zeitStempel);
			}
		}
	}
}
